/*	a r r a y c o n v s
	apl array conversion routines.
*/
#include <apl.h>
#include <stdlib.h>
#include <string.h>
#include <support.h>
#define	UNS	unsigned
#define	UCHAR	unsigned char

static	ARRAY	*dest;
static	BOUND	els;

/*	m k _ p t
	make an ARRAY, setting up dest (ptr), els (bnd).
*/
static void *mk_pt(UNS eltype, UNS rank, BOUND *pshape)
{
	els = arraybound(dest = mkarray(eltype, rank, pshape));
	return arraystart(dest);
}

/*	a g e t _ a
	make copy of array and return pointer to copy.
*/

static ARRAY *aget_a(ARRAY *src)
{
	size_t	bytes;
	
	dest = (ARRAY *)malloc(bytes = src->length * sizeof(MEMORYINT));
	
	if(dest)
		memcpy(dest, src, bytes);

	return(dest);
}

ARRAY	*sget_b(ARRAY *src) // get APLSINT array from APLBOOL array. 
{
	apluns8		*from;
	aplint8		*to;
	BOUND		bnd;
	apluns8		boolmask;

	from = arraystart(src);
	to = mk_pt(APLSINT, src->rank, src->shape);
	bnd = els;

	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*iget_b(ARRAY *src)	// get APLINTG array from APLBOOL array. 
{
	apluns8		*from;
	aplint16	*to;
	BOUND		bnd;
	apluns8		boolmask;

	from = arraystart(src);
	to = mk_pt(APLINTG, src->rank, src->shape);
	bnd = els;
	
	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*lget_b(ARRAY *src) // get APLLONG array from APLBOOL array. 
{
	apluns8		*from;
	aplint32	*to;
	BOUND		bnd;
	apluns8		boolmask;

	from = arraystart(src);
	to = mk_pt(APLLONG, src->rank, src->shape);
	bnd = els;
	
	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*dget_b(ARRAY *src) // get APLDOUB array from APLBOOL array. 
{
	apluns8		*from;
	aplfloat64	*to;
	BOUND		bnd;
	apluns8		boolmask;

	from = (UCHAR *)arraystart(src);
	to = (double *)mk_pt(APLDOUB, src->rank, src->shape);
	bnd = els;

	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

static ARRAY *iget_s(ARRAY *src) // get APLINTG array from APLSINT array. 
{
	aplint8		*from;
	aplint16	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLINTG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = *from++;

	return(dest);
}

static ARRAY *lget_s(ARRAY *src) // get APLLONG array from APLSINT array. 
{
	aplint8		*from;
	aplint32	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLLONG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = *from++;

	return(dest);
}

static ARRAY *dget_s(ARRAY *src) // get APLDOUB array from APLSINT array. 
{
	aplint8		*from;
	aplfloat64	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLDOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--)
		*to++ = *from++;

	return(dest);
}

static ARRAY *lget_i(ARRAY *src) // get APLLONG array from APLINTG array. 
{
	aplint16	*from;
	aplint32	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLLONG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = *from++;

	return(dest);
}

static ARRAY *dget_i(ARRAY *src) // get APLDOUB array from APLINTG array. 
{
	aplint16	*from;
	aplfloat64	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLDOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--)
		*to++ = *from++;

	return(dest);
}

static ARRAY *dget_l(ARRAY *src) // get APLDOUB array from APLLONG array. 
{
	aplint32	*from;
	aplfloat64	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLDOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--) 
		*to++ = *from++;

	return(dest);
}

static ARRAY *c8getc16(ARRAY *src) // get APLCHAR16 array from APLCHAR8 array. 
{
	aplWchar8	*from;
	aplWchar16	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLWCHAR16, src->rank, src->shape);
	bnd = els;

	while(bnd--) 
		*to++ = *from++;

	return(dest);
}

static ARRAY *c8getc32(ARRAY *src) // get APLCHAR32 array from APLCHAR8 array. 
{
	aplWchar8	*from;
	aplWchar32	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLWCHAR32, src->rank, src->shape);
	bnd = els;

	while(bnd--) 
		*to++ = *from++;

	return(dest);
}

static ARRAY *c16getc32(ARRAY *src) // get APLCHAR32 array from APLCHAR16 array. 
{
	aplWchar16	*from;
	aplWchar32	*to;
	BOUND		bnd;

	from = arraystart(src);
	to = mk_pt(APLWCHAR32, src->rank, src->shape);
	bnd = els;

	while(bnd--) 
		*to++ = *from++;

	return(dest);
}

static ARRAY *adomx(ARRAY *src)
	{
	domain();
	return 0; // keep compiler happy
	}

static ARRAY *anonx(ARRAY *src)
	{
	nonce();
	return 0; // keep compiler happy
	}

ARRAY	*(*convfns[NELTYPES][NELTYPES])(ARRAY *) =
{			// <to>get_<from>
// FROM     TO ---->
// |        CHAR   BOOL    SINT    INTG    LONG    DOUB    PNTR    WC8     WC16   WC32
/*CHAR	*/ {aget_a,adomx,  adomx,  adomx,  adomx,  adomx,  anonx  ,anonx, anonx, anonx},
/*BOOL	*/ {adomx, aget_a, sget_b, iget_b, lget_b, dget_b, anonx  ,adomx, adomx, adomx},
/*SINT	*/ {adomx, anonx , aget_a, iget_s, lget_s, dget_s, anonx  ,adomx, adomx, adomx},
/*INTG  */ {adomx, anonx , anonx , aget_a, lget_i, dget_i, anonx  ,adomx, adomx, adomx},
/*LONG	*/ {adomx, anonx , anonx , anonx , aget_a, dget_l, anonx  ,adomx, adomx, adomx},
/*DOUB	*/ {adomx, anonx , anonx , anonx , anonx , aget_a, anonx  ,adomx, adomx, adomx},
/*PNTR	*/ {anonx, anonx , anonx , anonx , anonx , anonx , anonx  ,anonx, anonx, anonx},
/*WC8	*/ {adomx, adomx,  adomx,  adomx,  adomx,  adomx,  anonx  ,aget_a,c8getc16, c8getc32},
/*WC16	*/ {adomx, adomx,  adomx,  adomx,  adomx,  adomx,  anonx  ,adomx, aget_a,c16getc32},
/*WC32	*/ {adomx, adomx,  adomx,  adomx,  adomx,  adomx,  anonx  ,adomx, adomx, aget_a}
};
